#    Openvpn Applet
#    Copyright (C) 2008  Mikko Vartiainen, mvartiainen@gmail.com
#
#    This program is free software: you can redistribute it and/or modify
#    it under the terms of the GNU General Public License as published by
#    the Free Software Foundation, either version 2 of the License, or
#    (at your option) any later version.
#
#    This program is distributed in the hope that it will be useful,
#    but WITHOUT ANY WARRANTY; without even the implied warranty of
#    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#    GNU General Public License for more details.
#
#    You should have received a copy of the GNU General Public License
#    along with this program.  If not, see <http://www.gnu.org/licenses/>.

import glob
import os
import string
from subprocess import Popen, PIPE, call
import time
import gtk
import hildondesktop
import hildon
import gobject

def pid_exists(pid):
    try:
        os.kill(pid, 0)
        return True
    except OSError, err:
        if err.errno == 1:
            return True
        else:
            return False
    
def openvpn_running(pid):
    p = Popen("pidof openvpn", shell=True, stdout=PIPE)
    pipe = p.stdout
    s = pipe.read().split()
    p.wait()
    print s
    if s.count(str(pid)):
        return True
    else:
        return False
        
class vpn():
    def __init__(self, conf_file):
        self.conf_file = conf_file
        self.name = string.split(os.path.basename(self.conf_file),'.')[0]
        self.pid_file = '/var/run/openvpn.'+self.name+'.pid'
        self.conf_dir = '/etc/openvpn'
                
    def get_status(self):
        pid = self.get_pid()
                
        if pid != None and pid_exists(pid) and openvpn_running(pid):
            #self.pid = pid
            return 0
        else:
            return -1
        
    def get_pid(self):
        print "pid_file: ",self.pid_file
        try:
            f = open(self.pid_file)
        except IOError:
            return None
        
        pid = int(f.readline())    
        return int(pid)
        
    def start(self):
        print "start"
        status = call(['sudo', '/etc/init.d/openvpn', 'start', self.name])
        return status
        
    def stop(self):
        print "stop"
        status = call(['sudo', '/etc/init.d/openvpn', 'stop', self.name])
        return status
        

class vpn_control():
    def read_vpn_conf(self, dir, ext):
        vpn_conf = []
        for e in ext:
            l = glob.glob(dir+'/*.'+e)
            if len(l) > 0: vpn_conf.extend(l)
        return vpn_conf        
        
    def __init__(self):
        self.vpn_list = []
        for v in self.read_vpn_conf('/etc/openvpn',['conf','ovpn']):
            print v
            self.vpn_list.append(vpn(v))
    
    def __str__(self):
        s = ''
        for i,v in enumerate(self.vpn_list):
            if v.get_status() != 0: cmd = 'start'
            else: cmd = 'stop'
            s = s+str(i+1)+'. '+v.name+' - '+cmd+'\n'
        return s   
        
class openvpn_applet(hildondesktop.HomeItem):
    def __init__(self):
        hildondesktop.HomeItem.__init__(self)
       
        self.vc = None
        self.banner = None
        self.timer = None       

        self.hbox = gtk.HBox()
        
        self.combo = gtk.combo_box_new_text()
        self.combo.connect("changed", self.combo_changed_event)
        self.hbox.pack_start(self.combo)
        self.update_list()
        
        self.vseparator = gtk.VSeparator()
        self.hbox.pack_start(self.vseparator)
        
        self.button = gtk.Button(label="Start")
        self.button.connect("clicked", self.button_clicked_event)
        self.hbox.pack_start(self.button)
        
        #self.set_resize_type(hildondesktop.HOME_ITEM_RESIZE_BOTH)
        self.add(self.hbox)

        self.hbox.show_all()
    
    def update_list(self):
        self.vc = vpn_control()
        for i,v in enumerate(self.vc.vpn_list):
            self.combo.insert_text(i,v.name)
        if len(self.vc.vpn_list) > 0:
            self.combo.set_active(0)
        
    def combo_changed_event(self,widget):
        index = self.combo.get_active()
        if index >= 0:
            if self.vc.vpn_list[index].get_status() != 0:
                self.button.set_label("Start")
            else:
                self.button.set_label("Stop")
    
    def button_clicked_event(self,widget):
        index = self.combo.get_active()
        self.timeout = 50
        self.timer = gobject.timeout_add(200, self.update_button)
        if index >= 0:
            if self.vc.vpn_list[index].get_status() != 0:
                self.banner = hildon.hildon_banner_show_information(self,None,"Starting Openvpn Connection")
                self.vc.vpn_list[index].start()
            else:
                self.banner = hildon.hildon_banner_show_information(self,None,"Stopping Openvpn Connection")
                self.vc.vpn_list[index].stop()
    
    def update_button(self):
        self.timeout = self.timeout - 1
        index = self.combo.get_active()
        if self.timeout > 0:
            if self.vc.vpn_list[index].get_status() != 0:
                new_label = "Start"
            else:
                new_label = "Stop"
            
            if self.button.get_label() != new_label:
                self.button.set_label(new_label)
                self.timeout = 0
                self.timer = None
            else:
                self.timer = gobject.timeout_add(200, self.update_button)
            
            
def hd_plugin_get_objects():
	plugin = openvpn_applet()
	return [plugin]


if __name__ == '__main__':
    vc = vpn_control()
    print vc
    sel = int(raw_input("command: "))-1
    if vc.vpn_list[sel].get_status() != 0:
        vc.vpn_list[sel].start()
    else:
        vc.vpn_list[sel].stop()
    
    
